/*============================================================================
	File: DBCCCommands.sql 

	Summary: This script shows using some undocumented DBCC commands 

	Date: October 2010 

	SQL Server Versions:
		10.0.2531.00 (SS2008 SP1)
		9.00.4035.00 (SS2005 SP3)
------------------------------------------------------------------------------
	Copyright (C) 2010 Paul S. Randal, SQLskills.com
	All rights reserved. 

	For more scripts and sample code, check out
		http://www.sqlskills.com/ 

	You may alter this code for your own *non-commercial* purposes. You may
	republish altered code as long as you give due credit. 

	THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF
	ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
	TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
	PARTICULAR PURPOSE.
============================================================================*/ 

USE master;
GO

IF DATABASEPROPERTYEX ('newdatabase', 'Version') > 0
	DROP DATABASE newdatabase;
GO
CREATE DATABASE newdatabase;
GO

-- Look at database metadata
DBCC DBINFO (newdatabase);
GO

-- Need the trace flag...
DBCC TRACEON (3604);
DBCC DBINFO (newdatabase);
GO

-- Run CHECKDB to update the last known good
DBCC CHECKDB (newdatabase) WITH NO_INFOMSGS, ALL_ERRORMSGS;
GO

DBCC DBINFO (newdatabase);
GO

-- Look at the log info
DBCC LOGINFO (newdatabase);
GO

-- Make the log file bigger
ALTER DATABASE newdatabase
	MODIFY FILE (
		NAME = newdatabase_Log,
		SIZE = 200MB,
		FILEGROWTH = 20MB);
GO

DBCC LOGINFO (newdatabase);
GO

-- How about forcing a log rebuild?
USE newdatabase;
GO
CREATE TABLE t1 (c1 int);
GO
BEGIN TRAN
GO
INSERT INTO t1 values (1);
GO
CHECKPOINT;
GO

-- In another window, shutdown...

-- Now remove the log file...

-- Restart SQL Server...

USE newdatabase;
GO


ALTER DATABASE newdatabase SET EMERGENCY;
ALTER DATABASE newdatabase SET SINGLE_USER;
GO

DBCC CHECKDB (newdatabase, REPAIR_ALLOW_DATA_LOSS)
WITH NO_INFOMSGS, ALL_ERRORMSGS;
GO

ALTER DATABASE newdatabase SET MULTI_USER;
GO

-- And now what does the DBINFO say?
DBCC TRACEON (3604);
DBCC DBINFO (newdatabase);
GO


