/*============================================================================
	File: Functions.sql 

	Summary: This script shows using some undocumented functions 

	Date: October 2010 

	SQL Server Versions:
		10.0.2531.00 (SS2008 SP1)
		9.00.4035.00 (SS2005 SP3)
------------------------------------------------------------------------------
	Copyright (C) 2010 Paul S. Randal, SQLskills.com
	All rights reserved. 

	For more scripts and sample code, check out
		http://www.sqlskills.com/ 

	You may alter this code for your own *non-commercial* purposes. You may
	republish altered code as long as you give due credit. 

	THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF
	ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
	TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
	PARTICULAR PURPOSE.
============================================================================*/ 

USE MASTER;
GO

IF DATABASEPROPERTYEX ('FNDBLogTest', 'Version') > 0
	DROP DATABASE FNDBLogTest;
GO

CREATE DATABASE FNDBLogTest;
GO

USE FNDBLogTest;
GO
SET NOCOUNT ON;
GO

-- Create a table to play with
CREATE TABLE ProdTable (
	c1 INT IDENTITY,
	c2 DATETIME DEFAULT GETDATE (),
	c3 AS c1 * 2 PERSISTED,
	c4 CHAR (5) DEFAULT 'a');
GO

INSERT INTO ProdTable DEFAULT VALUES;
GO 1000

-- Take initial backups
BACKUP DATABASE FNDBLogTest
TO DISK = 'D:\SQLskills\FNDBLogTest_Full.bak'
WITH INIT;
GO

BACKUP LOG FNDBLogTest
TO DISK = 'D:\SQLskills\FNDBLogTest_Log1.bak'
WITH INIT;
GO

-- Insert some more data
INSERT INTO ProdTable DEFAULT VALUES;
GO 1000

CREATE CLUSTERED INDEX Prod_CL
ON ProdTable (c1);
GO
CREATE NONCLUSTERED INDEX Prod_NCL
ON ProdTable (c2);
GO

INSERT INTO ProdTable DEFAULT VALUES;
GO 1000

-- How can we look in the log?
SELECT * FROM fn_dblog (NULL, NULL);
GO

-- Now do something specific
DROP INDEX Prod_NCL ON ProdTable;
GO

INSERT INTO ProdTable DEFAULT VALUES;
GO 1000

-- Can we find it?
SELECT * FROM fn_dblog (NULL, NULL);
GO

SELECT * FROM fn_dblog (NULL, NULL)
WHERE [Transaction Name] LIKE '%DROP%';
GO

SELECT * FROM fn_dblog (NULL, NULL)
WHERE [Transaction Id] = '0000:000011c1';
GO

-- Prove its a drop index on the table we're
-- interested in...
SELECT TOP (1) [Lock Information] FROM fn_dblog (NULL, NULL)
WHERE [Transaction Id] = '0000:000011c1'
AND [Lock Information] LIKE '%SCH_M OBJECT%';
GO

SELECT OBJECT_NAME ('2105058535');
GO

-- You could also find out which SPID did the drop and if
-- you're logging failed AND successful logins, you can see
-- who did the deed!

-- Now what if the log isn't there any more...
BACKUP LOG FNDBLogTest
TO DISK = 'D:\SQLskills\FNDBLogTest_Log2.bak'
WITH INIT;
GO

SELECT * FROM fn_dblog (NULL, NULL);
GO

-- Use the ability to return log from a backup...
SELECT * FROM fn_dump_dblog (
	NULL, NULL, 'DISK', 1, 'D:\SQLskills\FNDBLogTest_Log2.bak',
	DEFAULT,DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT,
	DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT,
	DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT,
	DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT,
	DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT,
	DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT,
	DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT,
	DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT,
	DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT);
GO

SELECT * FROM fn_dump_dblog (
	NULL, NULL, 'DISK', 1, 'D:\SQLskills\FNDBLogTest_Log2.bak',
	DEFAULT,DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT,
	DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT,
	DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT,
	DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT,
	DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT,
	DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT,
	DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT,
	DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT,
	DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT, DEFAULT)
WHERE [Transaction Name] LIKE '%DROP%';
GO

-- And so on...

-- Imagine that was a DROP TABLE command, we could
-- restore a copy of the database back to just before
-- the drop...

-- Restore full backup and all log backups....
RESTORE LOG FNDBLogTest_Copy
	FROM DISK = 'D:\SQLskills\FNDBLogTest_Log2.bak'
	WITH STOPBEFOREMARK = 'lsn:0000001f:00000134:0001',
	NORECOVERY, STATS;
GO