/*============================================================================
	File: Checkpoints.sql 

	Summary: This script shows checkpoint tracing using trace flags
	
	Date: October 2010 

	SQL Server Versions:
		10.0.2531.00 (SS2008 SP1)
		9.00.4035.00 (SS2005 SP3)
------------------------------------------------------------------------------
	Copyright (C) 2010 Paul S. Randal, SQLskills.com
	All rights reserved. 

	For more scripts and sample code, check out
		http://www.sqlskills.com/ 

	You may alter this code for your own *non-commercial* purposes. You may
	republish altered code as long as you give due credit. 

	THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF
	ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
	TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
	PARTICULAR PURPOSE.
============================================================================*/ 

USE MASTER;
GO

IF DATABASEPROPERTYEX ('CheckpointTest', 'Version') > 0
	DROP DATABASE CheckpointTest;

CREATE DATABASE CheckpointTest ON PRIMARY (
    NAME = 'CheckpointTest_data',
    FILENAME = N'D:\SQLskills\CheckpointTest_data.mdf')
LOG ON (
    NAME = 'CheckpointTest_log',
    FILENAME = N'D:\SQLskills\CheckpointTest_log.ldf',
    SIZE = 250MB,
    FILEGROWTH = 20MB);
GO

USE CheckpointTest;
GO
SET NOCOUNT ON;
GO

-- Create a table that will grow very
-- quickly and generate lots of transaction
-- log
CREATE TABLE BigRows (
	c1 INT IDENTITY,
	c2 CHAR (8000) DEFAULT 'a');
GO

-- Make sure the database is in SIMPLE
-- recovery model
ALTER DATABASE BadLog2008 SET RECOVERY SIMPLE;
GO

SET NOCOUNT ON;
GO

-- Trace CHECKPOINT execution
DBCC TRACEON (3605, -1);
DBCC TRACEON (3502, -1);
GO
EXEC sp_cycle_errorlog;
GO

INSERT INTO BigRows DEFAULT VALUES;
GO 20000

-- Look at errorlog at background task...

-- Now do manual checkpoint and look again
CHECKPOINT;
GO

-- Only shows start and end. What about details?
DBCC TRACEON (3504, -1);
GO

INSERT INTO BigRows DEFAULT VALUES;
GO 20000



-- Now do manual checkpoint and look again
CHECKPOINT;
GO