/*============================================================================
	File: Recompiles.sql 

	Summary: This script shows recompile tracing using trace flags
	
	Date: October 2010 

	SQL Server Versions:
		10.0.2531.00 (SS2008 SP1)
		9.00.4035.00 (SS2005 SP3)
------------------------------------------------------------------------------
	Copyright (C) 2010 Paul S. Randal, SQLskills.com
	All rights reserved. 

	For more scripts and sample code, check out
		http://www.sqlskills.com/ 

	You may alter this code for your own *non-commercial* purposes. You may
	republish altered code as long as you give due credit. 

	THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF
	ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
	TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
	PARTICULAR PURPOSE.
============================================================================*/

-- Cleanup from previous demo
DBCC TRACEOFF (3502, 3504, -1);
GO
EXEC sp_cycle_errorlog;
GO

USE MASTER;
GO

IF DATABASEPROPERTYEX ('RecompileTest', 'Version') > 0
	DROP DATABASE RecompileTest;
GO

CREATE DATABASE RecompileTest ON PRIMARY (
    NAME = 'RecompileTest_data',
    FILENAME = N'D:\SQLskills\RecompileTest_data.mdf')
LOG ON (
    NAME = 'RecompileTest_log',
    FILENAME = N'D:\SQLskills\RecompileTest_log.ldf',
    SIZE = 250MB,
    FILEGROWTH = 20MB);
GO

USE RecompileTest;
GO
SET NOCOUNT ON;
GO

-- Create a table to play with
CREATE TABLE ProdTable (
	c1 INT IDENTITY,
	c2 DATETIME DEFAULT GETDATE (),
	c3 AS c1 * 2 PERSISTED,
	c4 CHAR (5) DEFAULT 'a');
GO

INSERT INTO ProdTable DEFAULT VALUES;
GO 1000

-- Grab actual query plan in SSMS

SELECT COUNT (*) FROM ProdTable;
GO

-- Trace recompiles
DBCC TRACEON (3605, -1);
DBCC TRACEON (205, -1);
GO

CREATE CLUSTERED INDEX Prod_CL
ON ProdTable (c1);
GO

SELECT COUNT (*) FROM ProdTable;
GO

-- Look at errorlog using log viewer

/*
   Integer code   Reason for recompilation
   -----------------------------------------------------------------------
   1  		  Schema changed.
               	  
   2  		  Statistics changed.
      		  
   3  		  Recompile DNR.
      		  
   4  		  Set option changed.
      		  
   5  		  Temp table changed.
      		  
   6  		  Remote rowset changed.
      		  
   7  		  For browse perms changed.
      		  
   8  		  Query notification environment changed.
      		  
   9  		  MPI view changed.
      		  
   10 		  Cursor options changed.
      		  
   11 		  With recompile option.
*/


-- See all the crap in the there?

-- Use the undocumented XP instead...
EXEC xp_readerrorlog;
GO

-- Let's try another recompile
EXEC sp_cycle_errorlog;
GO

CREATE NONCLUSTERED INDEX Prod_NCL1
ON ProdTable (c3);
GO
SELECT COUNT (*) FROM ProdTable;
GO

EXEC xp_readerrorlog;
GO

-- Now trigger another recompile type
SELECT SUM (c3) FROM ProdTable
WHERE c1 > 1000
OPTION (RECOMPILE);
GO

EXEC xp_readerrorlog;
GO

-- And now another one
SELECT SUM (c1) FROM ProdTable
WHERE c1 <100000;
GO

-- Look at the plan - using stats to determine
-- that NC index is worth using

-- TURN OFF GETTING THE PLAN FIRST

-- Cause a statistics update
INSERT INTO ProdTable DEFAULT VALUES;
GO 5000

-- TURN BACK ON GETTING THE PLAN FIRST

SELECT SUM (c1) FROM ProdTable
WHERE c1 <100000;
GO

EXEC xp_readerrorlog;
GO





